/*                                                              */
/*                                                              */
/*  This file is free software, distributed under the terms     */
/*  of the GNU General Public License. See the file COPYING or  */
/*  http://www.fsf.org for more information.                    */
/*                                                              */
/*  runge_kutta    (c) Diane Neisius   (diane_va@yahoo.com)     */
/*                                                              */
/*                                                              */
/*--------------------------------------------------------------*/

/*                                                              */
/*              Runge-Kutta Verfahren 4.Ordnung                 */
/*                                                              */
/*
      erwartet folgende Voraussetzungen:

      - RUNGE_KUTTA_N mu in den Headerdateien oder im 
        Hauptprogramm auf die passende Dimension gesetzt 
        sein (geschieht in numerics.h)

      - Es mu eine Funktion definiert sein, in der die 
        DGL y'=f(x,y) spezifiziert ist. Sie wird erwartet 
        als

        void f( yy, x, y ), wobei  yy und y ein double 
                                   array der Dimension 
                                   RUNGE_KUTTA_N
                                   sind und x ein double.

        Der berechnete Wert von y' mu in yy bergeben 
        werden.

      - Der Aufruf des Verfahrens erfolgt dann mit

        runge_kutta( y_h, f, x, y, h ) , wobei h die 
                                               Schritt-
                                               weite ist.

        Der Wert y(x+h) wird in y_h zurckgegeben.

      11. 01.2002 ~Diane Neisius.

*/

# include "numerics.h"

void runge_kutta( double y_step[], 
                  void (*f)( double [], double, double [] ),
                  double x, double y[], double h )
   {
     double k1[RUNGE_KUTTA_N], k2[RUNGE_KUTTA_N], 
            k3[RUNGE_KUTTA_N], k4[RUNGE_KUTTA_N];
     double ybuf[RUNGE_KUTTA_N];
     int i;

     (*f)( k1, x, y);

     for ( i=0 ; i<RUNGE_KUTTA_N ; i++ )
        ybuf[i] = y[i] + h/2 * k1[i];
 
     (*f)( k2, x + h/2, ybuf );

     for ( i=0 ; i<RUNGE_KUTTA_N ; i++ )
        ybuf[i] = y[i] + h/2 * k2[i];
 
     (*f)( k3, x + h/2, ybuf );

     for ( i=0 ; i<RUNGE_KUTTA_N ; i++ )
        ybuf[i] = y[i] + h * k3[i];
 
     (*f)( k4, x + h, ybuf );

     for ( i=0 ; i<RUNGE_KUTTA_N ; i++ )
        y_step[i] = y[i] + h/6 * ( k1[i] + 2*k2[i] + 2*k3[i] + k4[i] );

   }





